@@ -10,10 +10,9 @@
 # [-o (show opencog XML output)] [-v verbose]
 # [-n parse-number] [--maxParses N] [--maxParseSeconds N]
 
-VM_OPTS="\
-	-Xmx1024m \
-	-Djava.library.path=/usr/lib:/usr/local/lib \
-	"
+export LANG=en_US.UTF-8
+
+VM_OPTS="-Xmx1024m"
 
 # By default these resources are read from the jar file or from 
 # the default paths (the same defined below)
@@ -25,10 +24,11 @@ VM_OPTS="\
 CLASSPATH="-classpath \
 bin:\
 /usr/local/share/java/jwnl.jar:\
-/usr/local/share/java/opennlp-tools-1.2.0.jar:\
+/usr/local/share/java/opennlp-tools-1.3.0.jar:\
 /usr/local/share/java/maxent-2.4.0.jar:\
 /usr/local/share/java/trove.jar:\
-/usr/local/share/java/link-grammar-4.3.4.jar:\
+/usr/local/share/java/link-grammar-4.3.5.jar:\
+/usr/share/java/link-grammar-4.3.5.jar:\
 /usr/share/java/commons-logging.jar:\
 /usr/share/java/gnu-getopt.jar:\
 /usr/share/java/xercesImpl.jar:\
