@@ -16,11 +16,16 @@
 
 package relex;
 
+import java.io.Serializable;
 import java.util.ArrayList;
 
 import relex.feature.Atom;
+import relex.feature.FeatureForeach;
 import relex.feature.FeatureNode;
+import relex.feature.FeatureNodeCallback;
 import relex.feature.LinkableView;
+import relex.feature.RelationCallback;
+import relex.feature.RelationForeach;
 import relex.stats.SimpleTruthValue;
 import relex.tree.PhraseTree;
 
@@ -42,8 +47,16 @@ import relex.tree.PhraseTree;
  *    likelihood of this parse of being a correct (meaningful) parse
  *    of the sentence.
  */
-public class ParsedSentence extends Atom
+public class ParsedSentence extends Atom implements Serializable
 {
+	private static final long serialVersionUID = -5518792541801263127L;
+
+	// Unique ID string identifying this parse.
+	private String idString;
+
+	// Back-pointer to collection of other parses for this sentence
+	private RelexInfo sentence;
+
 	// String containing the original sentence
 	private String original;
 
@@ -53,7 +66,7 @@ public class ParsedSentence extends Atom
 	// A string containing the Penn tree-bank style markup,
 	// aka "phrase structure" markup, for example
 	// (S (NP I) (VP am (NP a big robot)) .)
-	private String phraseSring;
+	private String phraseString;
 
 	private String errorString;
 
@@ -74,7 +87,7 @@ public class ParsedSentence extends Atom
 		original = originalString;
 		linkString = null;
 		errorString = "";
-		phraseSring = null;
+		phraseString = null;
 		leafConstituents = new ArrayList<FeatureNode>();
 	}
 
@@ -90,6 +103,22 @@ public class ParsedSentence extends Atom
 		return original;
 	}
 
+	public String getIDString() {
+		return idString;
+	}
+
+	public RelexInfo getRI() {
+		return sentence;
+	}
+
+	public void setRI(RelexInfo ri) {
+		sentence = ri;
+	}
+
+	public void setIDString(String str) {
+		idString = str;
+	}
+
 	public String getLinkString() {
 		return linkString;
 	}
@@ -99,11 +128,11 @@ public class ParsedSentence extends Atom
 	}
 
 	public String getPhraseString() {
-		return phraseSring;
+		return phraseString;
 	}
 
 	public void setPhraseString(String str) {
-		phraseSring = str;
+		phraseString = str;
 	}
 
 	public void setErrorString(String eString) {
@@ -115,22 +144,78 @@ public class ParsedSentence extends Atom
 	}
 
 	/* -------------------------------------------------------------------- */
-	public int getNumWords() {
+	public int getNumWords()
+	{
 		return leafConstituents.size();
 	}
 
-	public FeatureNode getWordAsNode(int i) {
+	/**
+	 * Return the i'th word in the sentence, as a feature node
+	 */
+	public FeatureNode getWordAsNode(int i)
+	{
 		return leafConstituents.get(i);
 	}
 
-	public String getWord(int i) {
+	/**
+	 * Return the i'th word in the sentence, as a string
+	 */
+	public String getWord(int i)
+	{
 		return LinkableView.getWordString(getWordAsNode(i));
 	}
 
-	public void addWord(FeatureNode w) {
+	public void addWord(FeatureNode w)
+	{
 		leafConstituents.add(w);
 	}
 
+	/**
+	 * Return feature node for the indicated word. Return null
+	 * if the word cannot be found in the sentence.  The input
+	 * word may be either the word as it appears in the sentence,
+	 * or its morphological root.
+	 *
+	 * If there are multiple occurances of a word in a sentence,
+	 * this will return only the left-most such occurance.
+	 */
+	public FeatureNode findWord(String word)
+	{
+		class word_cb implements FeatureNodeCallback
+		{
+			String match_word;
+			FeatureNode found;
+			word_cb(String mw)
+			{
+				match_word = mw;
+				found = null;
+			}
+
+			Boolean test(FeatureNode fn, FeatureNode fstr)
+			{
+				if (null == fstr) return false;
+				String w = fstr.getValue();
+				if (match_word.equals(w))
+				{
+					found = fn;
+					return true;
+				}
+				return false;
+			}
+			public Boolean FNCallback(FeatureNode fn)
+			{
+				Boolean rc = test(fn, fn.get("orig_str"));
+				if (rc) return rc;
+				rc = test(fn, fn.get("str"));
+				if (rc) return rc;
+				return false;
+			}
+		}
+		word_cb cb = new word_cb(word);
+		FeatureForeach.foreachWord(getLeft(), cb);
+		return cb.found;
+	}
+
 	/* -------------------------------------------------------------------- */
 	/* Various different views of the parsed sentence */
 
@@ -176,6 +261,20 @@ public class ParsedSentence extends Atom
 		return original;
 	}
 
+	/* ---------------------------------------------------------------- */
+	/**
+	 * Call the callback on each relation in the sentence
+	 */
+	public Boolean foreach(RelationCallback cb)
+	{
+		return RelationForeach.foreach(getLeft(), cb);
+	}
+	
+	public Boolean foreach(FeatureNodeCallback cb)
+	{
+		return RelationForeach.foreach(getLeft(), cb);
+	}
+
 	/* ---------------------------------------------------------------- */
 	/**
 	 * @return the FeatureNode representing the left-most word in the sentence.
@@ -194,6 +293,7 @@ public class ParsedSentence extends Atom
 	}
 
 	/* ---------------------------------------------------------------- */
+	/* Return unpacked meta information about parse, and ranking too */
 
 	public int getAndCost()
 	{
@@ -227,6 +327,11 @@ public class ParsedSentence extends Atom
 	 * Perform a crude parse-ranking based on Link-grammar output.
 	 * The ranking will be stored as the "confidence" of the 
 	 * TruthValue associated with this parse.
+	 *
+	 * A classic example of competing parses for a sentence is:
+	 * (S (NP I) (VP saw (NP the man) (PP with (NP the binoculars))) .)
+	 * (S (NP I) (VP saw (NP (NP the man) (PP with (NP the binoculars)))) .)
+	 * The ranker below gives both about equal scores.
 	 */
 	public void simpleRankParse()
 	{
@@ -247,6 +352,23 @@ public class ParsedSentence extends Atom
 
 		stv.setConfidence(weight);
 	}
+	
+	public int hashCode()
+	{
+		if (original == null)
+			return 0;
+		return original.hashCode() | leafConstituents.size();
+	}
 
+	public boolean equals(Object x)
+	{
+		if (! (x instanceof ParsedSentence))
+			return false;
+		ParsedSentence p = (ParsedSentence)x;
+		if (original == null)
+			return p.original == null;
+		else
+			return original.equals(p.original) && this.leafConstituents.equals(p.leafConstituents);
+	}
+	
 } // end ParsedSentence
-
