@@ -32,9 +32,10 @@ import relex.chunk.PatternChunker;
 import relex.chunk.PhraseChunker;
 import relex.chunk.RelationChunker;
 import relex.concurrent.RelexContext;
+import relex.corpus.EntityMaintainerFactory;
+// import relex.corpus.QuotesParensSentenceDetector;
 import relex.corpus.DocSplitter;
 import relex.corpus.DocSplitterFactory;
-import relex.corpus.GateEntityMaintainer;
 import relex.entity.EntityInfo;
 import relex.entity.EntityMaintainer;
 import relex.feature.LinkView;
@@ -49,22 +50,27 @@ import relex.parser.LinkParser;
 import relex.parser.LinkParserClient;
 import relex.parser.LinkParserJNINewClient;
 import relex.parser.LinkParserSocketClient;
+import relex.stats.TruthValue;
+import relex.stats.SimpleTruthValue;
 import relex.tree.PhraseMarkup;
+import relex.tree.PhraseTree;
 
 /**
  * The RelationExtractor class provides the central processing
- * point for parsing sentences and extracting semantic 
+ * point for parsing sentences and extracting semantic
  * relationships from them.  The main() proceedure is usable
  * as a stand-alone document analyzer; it supports several
  * flags modifying the displayed output.
  *
  * The primarey interface is the processSentence() method,
  * which accepts one sentence at a time, parses it, and extracts
- * relationships from it. This method is stateful: it also 
+ * relationships from it. This method is stateful: it also
  * performs anaphora resolution.
  */
 public class RelationExtractor
 {
+	public static final int verbosity = 1;
+
 	public static final int DEFAULT_MAX_PARSES = 100;
 	public static final int DEFAULT_MAX_SENTENCE_LENGTH = 1024;
 	public static final int DEFAULT_MAX_PARSE_SECONDS = 30;
@@ -72,20 +78,24 @@ public class RelationExtractor
 	public static final String DEFAULT_ALGS_FILE = "./data/relex-semantic-algs.txt";
 
 	/** The LinkParserClient to be used - this class isn't thread safe! */
-	private RelexContext context; 
-	
+	private RelexContext context;
+
 	/** Syntax processing */
 	private LinkParser parser;
 
 	/** Semantic processing */
 	private SentenceAlgorithmApplier sentenceAlgorithmApplier;
-	
+
 	/** Penn tree-bank style phrase structure markup. */
 	private PhraseMarkup phraseMarkup;
 
 	/** Anaphora resolution */
 	public Antecedents antecedents;
 	private Hobbs hobbs;
+	public boolean do_anaphora_resolution;
+
+	/** Statistics */
+	private ParseStats stats;
 
 	/* ---------------------------------------------------------- */
 	/* Constructors, etc. */
@@ -93,14 +103,14 @@ public class RelationExtractor
 	public RelationExtractor(boolean useSocket)
 	{
 		parser = new LinkParser();
-		
+
 		LinkParserClient lpc = (useSocket) ? new LinkParserSocketClient() : LinkParserJNINewClient.getSingletonInstance();
 		lpc.init();
 		Morphy morphy = MorphyFactory.getImplementation(MorphyFactory.DEFAULT_SINGLE_THREAD_IMPLEMENTATION);
 		context = new RelexContext(lpc, morphy);
-		
+
 		sentenceAlgorithmApplier = new SentenceAlgorithmApplier();
-		
+
 		setMaxParses(DEFAULT_MAX_PARSES);
 		setMaxParseSeconds(DEFAULT_MAX_PARSE_SECONDS);
 		setMaxCost(DEFAULT_MAX_PARSE_COST);
@@ -108,6 +118,9 @@ public class RelationExtractor
 		phraseMarkup = new PhraseMarkup();
 		antecedents = new Antecedents();
 		hobbs = new Hobbs(antecedents);
+		do_anaphora_resolution = false;
+
+		stats = new ParseStats();
 	}
 
 	/* ---------------------------------------------------------- */
@@ -149,7 +162,7 @@ public class RelationExtractor
 		hobbs = new Hobbs(antecedents);
 	}
 
-	public RelexInfo processSentence(String sentence) 
+	public RelexInfo processSentence(String sentence)
 	{
 		return processSentence(sentence, null);
 	}
@@ -157,15 +170,16 @@ public class RelationExtractor
 	public RelexInfo processSentence(String sentence,
 	                                 EntityMaintainer entityMaintainer)
 	{
+		starttime = System.currentTimeMillis();
 		if (entityMaintainer == null)
 		{
-			entityMaintainer = new EntityMaintainer(sentence, 
+			entityMaintainer = new EntityMaintainer(sentence,
 		                               new ArrayList<EntityInfo>());
 		}
 
 		RelexInfo ri = parseSentence(sentence, entityMaintainer);
 
-		for (ParsedSentence parse : ri.parsedSentences)
+		for (ParsedSentence parse : ri.getParses())
 		{
 			// Markup feature node graph with entity info,
 			// so that the relex algs (next step) can see them.
@@ -174,20 +188,28 @@ public class RelationExtractor
 			// The actual relation extraction is done here.
 			sentenceAlgorithmApplier.applyAlgs(parse, context);
 
-			// Strip out the entity markup, so that when the 
+			// Strip out the entity markup, so that when the
 			// sentence is printed, we don't print gunk.
 			entityMaintainer.repairSentence(parse.getLeft());
 
 			// Also do a Penn tree-bank style phrase structure markup.
 			phraseMarkup.markup(parse);
 
+			// Repair the entity-mangled tree-bank string.
+			PhraseTree pt = new PhraseTree(parse.getLeft());
+			parse.setPhraseString(pt.toString());
+
 			// Assign a simple parse-ranking score, based on LinkGrammar data.
 			parse.simpleRankParse();
 		}
 
 		// Perform anaphora resolution
-		hobbs.addParse(ri);
-		hobbs.resolve(ri);
+		if (do_anaphora_resolution)
+		{
+			hobbs.addParse(ri);
+			hobbs.resolve(ri);
+		}
+		if (verbosity > 0) reportTime("Relex processing: ");
 		return ri;
 	}
 
@@ -204,14 +226,14 @@ public class RelationExtractor
 		}
 		if (sentence == null) return null;
 
-		ArrayList<ParsedSentence> parses = null;
+		String orig_sentence = entityMaintainer.getOriginalSentence();
+		RelexInfo ri = null;
 		if (sentence.length() < DEFAULT_MAX_SENTENCE_LENGTH) {
-			parses = parser.parse(sentence, context.getLinkParserClient());
+			ri = parser.parse(sentence, context.getLinkParserClient());
 		} else {
-			System.err.println("Sentence too long!: " + sentence);
-			parses = new ArrayList<ParsedSentence>();
+			ri = new RelexInfo();
 		}
-		RelexInfo ri = new RelexInfo(sentence, parses);
+		ri.setSentence(orig_sentence);
 		return ri;
 	}
 
@@ -238,24 +260,46 @@ public class RelationExtractor
 		System.out.println("\n======\n");
 	}
 
+	// Punish chunks whose length is other than 3.
+	private static void discriminate(ChunkRanker ranker)
+	{
+		ArrayList<LexChunk> chunks = ranker.getChunks();
+		for (LexChunk ch : chunks)
+		{
+			int sz = ch.size();
+			double weight = sz-3;
+			if (weight < 0) weight = - weight;
+			weight = 1.0 - 0.2 * weight;
+
+			// twiddle the confidence of the chunk
+			TruthValue tv = ch.getTruthValue();
+			SimpleTruthValue stv = (SimpleTruthValue) tv;
+			double confidence = stv.getConfidence();
+			confidence *= weight;
+			stv.setConfidence(confidence);
+		}
+	}
+
 	/* ---------------------------------------------------------- */
 	/**
 	 * Main entry point
 	 */
-	public static void main(String[] args) 
+	public static void main(String[] args)
 	{
-		String callString = "RelationExtractor" + 
+		String callString = "RelationExtractor" +
+			" [-a (perform anaphora resolution)]" +
 			" [-c (show plain output)]" +
 			" [-f (show frame output)]" +
 			" [-g (use GATE entity detector)]" +
 			" [-h (show this help)]" +
 			" [-l (show parse links)]" +
 			" [-m (show parse metadata)]" +
-			" [-n parse-number]" +
+			" [-n max number of parses to display]" +
 			" [-o (show opencog XML output)]" +
 			" [--pa (show phrase-based lexical chunks)]" +
 			" [--pb (show pattern-based lexical chunks)]" +
 			" [--pc (show relational lexical chunks)]" +
+			" [-q (do NOT show relations)]" +
 			" [-r (show raw output)]" +
 			" [-s Sentence (in quotes)]" +
 			" [-t (show parse tree)]" +
@@ -263,6 +307,7 @@ public class RelationExtractor
 			" [-x (show cerego XML output)]" +
 			" [--maxParseSeconds N]";
 		HashSet<String> flags = new HashSet<String>();
+		flags.add("-a");
 		flags.add("-c");
 		flags.add("-f");
 		flags.add("-g");
@@ -273,6 +318,7 @@ public class RelationExtractor
 		flags.add("--pa");
 		flags.add("--pb");
 		flags.add("--pc");
+		flags.add("-q");
 		flags.add("-r");
 		flags.add("-t");
 		flags.add("-v");
@@ -284,16 +330,16 @@ public class RelationExtractor
 		Map<String,String> commandMap = CommandLineArgParser.parse(args, opts, flags);
 
 		String sentence = null;
-		int maxParses = 10;
+		int maxParses = 30;
 		int maxParseSeconds = 60;
 
 		// Check for optional command line arguments.
 		try
 		{
-			maxParses = commandMap.get("-n") != null ? 
+			maxParses = commandMap.get("-n") != null ?
 				Integer.parseInt(commandMap.get("-n").toString()) : 1;
 
-			sentence = commandMap.get("-s") != null ? 
+			sentence = commandMap.get("-s") != null ?
 				commandMap.get("-s").toString() : null;
 
 			maxParseSeconds = commandMap.get("--maxParseSeconds") != null ?
@@ -322,33 +368,44 @@ public class RelationExtractor
 		re.setMaxParses(maxParses);
 		re.setMaxParseSeconds(maxParseSeconds);
 
-		GateEntityMaintainer gem = null;
+		if (commandMap.get("-a") != null)
+		{
+			re.do_anaphora_resolution = true;
+		}
+
+		EntityMaintainerFactory gem = null;
 		if (commandMap.get("-g") != null)
 		{
 			re.starttime = System.currentTimeMillis();
-			gem = new GateEntityMaintainer();
-			gem.initialize();
-			re.reportTime("Gate initialization: ");
+			gem = EntityMaintainerFactory.get();
+			gem.makeEntityMaintainer(""); // force initialization to measure initialization time
+			re.reportTime("Entity Detection Initialization: ");
 		}
 
 		// If sentence is not passed at command line, read from standard input:
 		BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
 		DocSplitter ds = DocSplitterFactory.create();
 
+		// QuotesParens is currently broken, it fails to handle possesives.
+		// QuotesParensSentenceDetector ds = QuotesParensSentenceDetector.create();
+
 		ParseView ceregoView = new ParseView();
 		OpenCogXML opencog = new OpenCogXML();
-		Frame frame = new Frame();
 
+		Frame frame = null;
+		if (commandMap.get("-f") != null) frame = new Frame();
+
+		int sentence_count = 0;
 		while(true)
 		{
-			// If no sentence specified on the command line 
+			// If no sentence specified on the command line
 			// (with the "-s" flag), then read it from stdin.
 			while (sentence == null)
 			{
 				System.out.print("% ");
 				try {
 					sentence = stdin.readLine();
-					if ((sentence == null) || "END.".equals(sentence)) 
+					if ((sentence == null) || "END.".equals(sentence))
 					{
 						System.out.println("Bye.");
 						if (commandMap.get("-o") != null)
@@ -364,7 +421,7 @@ public class RelationExtractor
 					System.err.println("Error reading sentence from the standard input!");
 				}
 
-				// Buffer up input text, and wait for a whole, 
+				// Buffer up input text, and wait for a whole,
 				// complete sentence before continuing.
 				ds.addText(sentence + " ");
 				sentence = ds.getNextSentence();
@@ -372,32 +429,41 @@ public class RelationExtractor
 
 			while (sentence != null)
 			{
+				System.out.println("SENTENCE: ["+sentence+"]");
 				EntityMaintainer em = null;
 				if (gem != null)
 				{
 					re.starttime = System.currentTimeMillis();
-					em = gem.process(sentence);
+					em = gem.makeEntityMaintainer(sentence);
 					re.reportTime("Gate processing: ");
 				}
 
-				re.starttime = System.currentTimeMillis();
 				RelexInfo ri = re.processSentence(sentence,em);
-				re.reportTime("Relex processing: ");
-	
-				int np = ri.parsedSentences.size();
+
+				sentence_count ++;
+				re.stats.bin(ri);
+
+				int np = ri.getParses().size();
 				if (np > maxParses) np = maxParses;
-				ChunkRanker ranker = new ChunkRanker(np);
+
+				// chunk ranking stuff
+				ChunkRanker ranker = new ChunkRanker();
+				double parse_weight = 1.0 / ((double) np);
+				double votes = 1.0e-20;
+				if (commandMap.get("--pa") != null) votes += 1.0;
+				if (commandMap.get("--pb") != null) votes += 2.0;
+				if (commandMap.get("--pc") != null) votes += 1.0;
+				votes = 1.0 / votes;
+				votes *= parse_weight;
 
 				// Print output
 				int numParses = 0;
-				for (ParsedSentence parse: ri.parsedSentences)
+				for (ParsedSentence parse: ri.getParses())
 				{
-					ChunkRanker rollup = new ChunkRanker(1);
-
 					System.out.println(sentence);
 					System.out.println("\n====\n");
-					System.out.println("Parse " + (numParses+1) + 
-					             " of " + ri.parsedSentences.size());
+					System.out.println("Parse " + (numParses+1) +
+					             " of " + ri.getParses().size());
 
 					if (commandMap.get("-r") != null)
 					{
@@ -405,7 +471,7 @@ public class RelationExtractor
 						System.out.println(RawView.printZHeads(parse.getLeft()));
 						System.out.println("\n======\n");
 					}
-	
+
 					if (commandMap.get("-t") != null)
 						System.out.println("\n" + parse.getPhraseString());
 
@@ -434,17 +500,20 @@ public class RelationExtractor
 					if (commandMap.get("-v") != null)
 						System.out.println("\n" + parse.getLeft().toString(LinkView.getFilter()));
 
-					System.out.println("\n======\n");
-					System.out.println(SimpleView.printRelations(parse.getLeft()));
-					System.out.println("\n======\n");
-	
+					if (commandMap.get("-q") == null)
+					{
+						System.out.println("\n======\n");
+						System.out.println(SimpleView.printRelations(parse));
+						System.out.println("\n======\n");
+					}
+
 					if (commandMap.get("--pa") != null)
 					{
 						System.out.println("Phrase tree-based lexical chunks:");
 						LexicalChunker chunker = new PhraseChunker();
 						chunker.findChunks(parse);
 						prt_chunks(chunker.getChunks());
-						rollup.add(chunker.getChunks(), parse.getTruthValue());
+						ranker.add(chunker.getChunks(), parse.getTruthValue(), votes);
 					}
 					if (commandMap.get("--pb") != null)
 					{
@@ -452,7 +521,7 @@ public class RelationExtractor
 						LexicalChunker chunker = new PatternChunker();
 						chunker.findChunks(parse);
 						prt_chunks(chunker.getChunks());
-						rollup.add(chunker.getChunks(), parse.getTruthValue());
+						ranker.add(chunker.getChunks(), parse.getTruthValue(), 2.0*votes);
 					}
 					if (commandMap.get("--pc") != null)
 					{
@@ -460,13 +529,9 @@ public class RelationExtractor
 						LexicalChunker chunker = new RelationChunker();
 						chunker.findChunks(parse);
 						prt_chunks(chunker.getChunks());
-						rollup.add(chunker.getChunks(), parse.getTruthValue());
+						ranker.add(chunker.getChunks(), parse.getTruthValue(), votes);
 					}
 
-					// Now roll them up. Need to do this in two stages
-					// to get the probability calculations correct.
-					ranker.add(rollup.getChunks(), parse.getTruthValue());
-
 					if (commandMap.get("-c") != null)
 					{
 						ceregoView.setParse(parse);
@@ -485,7 +550,7 @@ public class RelationExtractor
 					if (commandMap.get("-f") != null)
 					{
 						re.starttime = System.currentTimeMillis();
-						String fin = SimpleView.printRelationsAlt(parse.getLeft());
+						String fin = SimpleView.printRelationsAlt(parse);
 						String[] fout = frame.process(fin);
 						re.reportTime("Frame processing: ");
 						for (int i=0; i < fout.length; i++) {
@@ -509,14 +574,26 @@ public class RelationExtractor
 
 					if (++numParses >= maxParses) break;
 				}
+
 				if (0 < ranker.getChunks().size())
 				{
+					discriminate(ranker);
 					System.out.println("\nLexical Chunks:\n" +
 					             ranker.toString());
 				}
-				System.out.println("\nAntecedent candidates:\n"
-				                   + re.antecedents.toString());
-	
+
+				if (re.do_anaphora_resolution)
+				{
+					System.out.println("\nAntecedent candidates:\n"
+					                   + re.antecedents.toString());
+				}
+
+				// Print out the stats every now and then.
+				if (sentence_count%5 == 0)
+				{
+					System.out.println ("\n" + re.stats.toString());
+				}
+
 				sentence = ds.getNextSentence();
 			}
 			if (commandMap.get("-s") != null) break;
