@@ -35,7 +35,7 @@ import relex.tree.PhraseTree;
 
 public class Hobbs
 {
-	public static final int DEBUG = 0; 
+	public static final int DEBUG = 0;
 	
 	// Buffer of sentences previously seen.
 	private ArrayList<RelexInfo> sentences;
@@ -78,7 +78,7 @@ public class Hobbs
 
 	public void resolve(RelexInfo ri)
 	{
-		ArrayList<ParsedSentence> plist = ri.parsedSentences;
+		ArrayList<ParsedSentence> plist = ri.getParses();
 		if (0 >= plist.size()) return;
 
 		// XXX at the moment, ignore all but the first parse.
@@ -209,10 +209,10 @@ public class Hobbs
 			RelexInfo ri = sentences.get(i);
 
 			// Whoops . sentence had zero parses!
-			if (ri.parsedSentences.size() == 0) continue;
+			if (ri.getParses().size() == 0) continue;
 
 			// XXX ignore all but the first parse right now
-			ParsedSentence ps = ri.parsedSentences.get(0);
+			ParsedSentence ps = ri.getParses().get(0);
 			PhraseTree head = ps.getPhraseTree();
 			StepFour(head);
 		}
@@ -406,12 +406,15 @@ public class Hobbs
 
 						antecedents.add(anaphore, pt);
 						if (DEBUG > 0)
-						System.out.println("found antecedent "
+						{
+							System.out.println("Found antecedent "
 						                 +  pt.toString()
 						                 + " to pronoun "
 						                 + anaphore.toString());
 
-						// System.out.println("List:\n" + antecedents.toString(anaphore));
+							System.out.println("Current list:\n"
+							              + antecedents.toString(anaphore));
+						}
 						if (stop_at_np) pt.setMark(marker);
 					}
 				}
