@@ -15,9 +15,12 @@
  */
 package relex.entity;
 
+import java.io.Serializable;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.List;
 import java.util.TreeSet;
 
 import relex.feature.FeatureNode;
@@ -25,20 +28,23 @@ import relex.feature.LinkableView;
 import relex.feature.SemanticView;
 import relex.output.RawView;
 
-public class EntityMaintainer
+public class EntityMaintainer implements Serializable 
 {
+	private static final long serialVersionUID = -8186219027158709715L;
+
 	/**
 	 * This number is derived from the LinkParser dictionary -- 
 	 * it only accepts suffixes up to "60." To increase this 
-	 * number, you need to add more words to the Link parser 
-	 * dictionary, like "genericID61, dateID61, etc."
+	 * number, you need to add more words to the Link Parser 
+	 * dictionary, like "genericID61", "dateID61", etc.
 	 */
 	public static int MAX_NUM_ENTITIES = 60;
 
 	// The original sentence string
 	private String originalSentence;
 
-	public String getOriginalSentence() {
+	public String getOriginalSentence()
+	{
 		return originalSentence;
 	}
 
@@ -46,7 +52,8 @@ public class EntityMaintainer
 	// except all entities are replaced with ID strings
 	private String convertedSentence;
 
-	public String getConvertedSentence() {
+	public String getConvertedSentence()
+	{
 		return convertedSentence;
 	}
 
@@ -69,6 +76,7 @@ public class EntityMaintainer
 
 	static
 	{
+		// Emoticons -- smiley faces, right :-)
 		// Partial list, only of the basics, taken from wikipedia
 		// This could be improved on by automatically generating
 		// these with and without noses, etc. 
@@ -221,21 +229,36 @@ public class EntityMaintainer
 					new Integer(convertedSentence.length() - 1));
 			}
 
-			// If the entity is at the end of the sentence, then the 
-			// entity detector may swallow the period at the end 
-			// of the sentence. This can confuse downstream handlers,
-			// so remove the period from the entity, and put it back 
-			// at the end of the sentence.
-			if ((originalSentence.length() == curIndex) &&
+			curIndex = eInfo.getLastCharIndex();
+
+			// If the entity ends with a period, then sometimes the
+			// entity detector doesn't include the period as part of 
+			// the entity. Fix this up (as long as the period is not
+			// at the enbd of the sentence).
+			// For example: "The A.D.A. advises against this."
+			if ((originalSentence.length() > curIndex) &&
 			    (originalSentence.charAt(eInfo.getLastCharIndex()) == '.'))
 			{
-				eInfo.setLastCharIndex(eInfo.getLastCharIndex()-1);
+				eInfo.setLastCharIndex(eInfo.getLastCharIndex()+1);
+				curIndex++;
 			}
 
 			// Insert the ID string of the entity.
-			curIndex = eInfo.getLastCharIndex();
 			convertedSentence += makeID(eInfo);
 
+			// If the entity is at the end of the sentence, then the 
+			// entity detector may swallow the period at the end 
+			// of the sentence. This can confuse downstream handlers,
+			// so add a new period to end the sentence.
+			// For example: "It is located in Wshington, D.C."
+			if ((originalSentence.length() == curIndex) &&
+			    (originalSentence.charAt(eInfo.getLastCharIndex()-1) == '.'))
+			{
+				convertedSentence += '.';
+				insertedWhitespaceCharIndexes.add(
+					new Integer(convertedSentence.length() - 1));
+			}
+
 			// Insert trailing white space after eInfo if it is 
 			// not followed by a legal string
 			if (originalSentence.length() > curIndex
@@ -251,6 +274,7 @@ public class EntityMaintainer
 		// Copy the remaining portion (if any) of the original string.
 		if (curIndex < originalSentence.length())
 			convertedSentence += originalSentence.substring(curIndex);
+
 		// System.out.println("CONVERSION\n" + originalSentence + "\n" +
 		// convertedSentence);
 	}
@@ -275,7 +299,7 @@ public class EntityMaintainer
 			if (start < 0) continue;
 			int end = start + emo.length();
 	
-			EntityInfo ei = new EmoticonEntityInfo(originalSentence, start, end);
+			EntityInfo ei = new EntityInfo(originalSentence, start, end, EntityType.EMOTICON);
 			addEntity(ei);
 		}
 	}
@@ -294,7 +318,7 @@ public class EntityMaintainer
 			start = originalSentence.indexOf('(', start);
 			if (start < 0) break;
 
-			EntityInfo ei = new PunctuationEntityInfo(originalSentence, start, start+1);
+			EntityInfo ei = new EntityInfo(originalSentence, start, start+1, EntityType.PUNCTUATION);
 			addEntity(ei);
 			start++;
 		}
@@ -303,7 +327,7 @@ public class EntityMaintainer
 			start = originalSentence.indexOf(')', start);
 			if (start < 0) break;
 
-			EntityInfo ei = new PunctuationEntityInfo(originalSentence, start, start+1);
+			EntityInfo ei = new EntityInfo(originalSentence, start, start+1, EntityType.PUNCTUATION);
 			addEntity(ei);
 			start++;
 		}
@@ -340,8 +364,7 @@ public class EntityMaintainer
 	/*
 	 * CONSTRUCTOR
 	 */
-	public EntityMaintainer(String _originalSentence,
-			ArrayList<EntityInfo> eis)
+	public EntityMaintainer(String _originalSentence, Collection<EntityInfo> eis)
 	{
 		if (eis.size() > MAX_NUM_ENTITIES)
 		{
@@ -374,11 +397,13 @@ public class EntityMaintainer
 
 	// --------------------------------------------------------
 
-	public boolean isEntityID(String e) {
+	public boolean isEntityID(String e)
+	{
 		return iDs2Entities.containsKey(e);
 	}
 
-	public EntityInfo getEntityInfo(String entityID) {
+	public EntityInfo getEntityInfo(String entityID)
+	{
 		return iDs2Entities.get(entityID);
 	}
 
@@ -401,30 +426,40 @@ public class EntityMaintainer
 	public void repairSentence(FeatureNode leftNode)
 	{
 		int charDelta = 0;
-		try {
-		for (LinkableView word = new LinkableView(leftNode); word != null; word = (word
-				.getNext() == null ? null : new LinkableView(word.getNext())))
+		try
 		{
-			int previousWhiteSpace = previouslyInsertedWhitespace(word
-					.getStartChar());
-			String wordName = word.getWordString();
-			word.setStartChar(word.getStartChar() + charDelta
-					- previousWhiteSpace);
-			// word.setExpandedStartChar(word.getExpandedStartChar()+charDelta);
-			if (isEntityID(wordName)) {
-				EntityInfo entInfo = getEntityInfo(wordName);
-				String origName = entInfo.getOriginalString();
-				charDelta += origName.length() - wordName.length();
-				try {
-					SemanticView semView = new SemanticView(word.fn()
-							.get("ref"));
-					semView.setName(origName);// .replaceAll("\\s","_"))
-				} catch (Exception e) {
+			for (LinkableView word = new LinkableView(leftNode);
+			     word != null;
+			     word = (word.getNext() == null ?
+			                  null : new LinkableView(word.getNext())))
+			{
+				int previousWhiteSpace = 
+				      previouslyInsertedWhitespace(word.getStartChar());
+
+				String wordName = word.getWordString();
+
+				word.setStartChar(word.getStartChar() + charDelta
+						- previousWhiteSpace);
+
+				// word.setExpandedStartChar(word.getExpandedStartChar()+charDelta);
+
+				if (isEntityID(wordName))
+				{
+					EntityInfo entInfo = getEntityInfo(wordName);
+					String origName = entInfo.getOriginalString();
+					charDelta += origName.length() - wordName.length();
+					try
+					{
+						SemanticView semView = new SemanticView(word.fn()
+								.get("ref"));
+						semView.setName(origName);
+					}
+					catch (Exception e) {}
 				}
+
+				word.setEndChar(word.getEndChar() + charDelta - previousWhiteSpace);
+				// word.setExpandedEndChar(word.getExpandedEndChar()+charDelta);
 			}
-			word.setEndChar(word.getEndChar() + charDelta - previousWhiteSpace);
-			// word.setExpandedEndChar(word.getExpandedEndChar()+charDelta);
-		}
 		}
 		catch (Exception e)
 		{
@@ -435,6 +470,15 @@ public class EntityMaintainer
 		}
 	}
 
+	/**
+	 * Return all EntityInfo's ordered by their starting character
+	 * position.
+	 */
+	public List<EntityInfo> getEntities()
+	{
+		return orderedEntityInfos;
+	}
+	
 	/**
 	 * prepareSentence() -- markup parsed sentence with entity 
 	 * information. This needs to be done before the relex algs run,
@@ -442,20 +486,25 @@ public class EntityMaintainer
 	 */
 	public void prepareSentence(FeatureNode leftNode)
 	{
-		for (LinkableView word = new LinkableView(leftNode); word != null; word = (word
-				.getNext() == null ? null : new LinkableView(word.getNext())))
+		for (LinkableView word = new LinkableView(leftNode); 
+		     word != null; 
+		     word = (word.getNext() == null ? 
+		                null : new LinkableView(word.getNext())))
 		{
 			String wordName = word.getWordString();
-			if (isEntityID(wordName)) {
+			if (isEntityID(wordName))
+			{
 				EntityInfo entInfo = getEntityInfo(wordName);
 				entInfo.setProperties(word.fn());
 			}
 		}
 	}
 
-	public String toString(){
+	public String toString()
+	{
 		StringBuilder sb = new StringBuilder();
-		for (EntityInfo info: orderedEntityInfos){
+		for (EntityInfo info: orderedEntityInfos)
+		{
 			String name = info.getOriginalSentence().substring(
 					info.getFirstCharIndex(), 
 					info.getLastCharIndex());
@@ -464,18 +513,19 @@ public class EntityMaintainer
 		return sb.toString();
 	}
 	
+	/**
+	 * Arg0: a sentence with an entity Arg1: the first character of the
+	 * entity Arg2: the last character of the entity Example:
+	 * java relex.entity.EntityMaintainer "Does Mike think it will work?" 5 8
+	 * Expected output: Does genericID1 think it will work?
+	 */
 	public static void main(String[] args)
 	{
-		/*
-		 * Arg0: a sentence with an entity Arg1: the first character of the
-		 * entity Arg2: the last character of the entity Example:
-		 * java relex.entity.EntityMaintainer "Does Mike think it will work?" 5 8
-		 * Expected output: Does genericID1 think it will work?
-		 */
 		String sentence = args[0];
 		ArrayList<EntityInfo> list = new ArrayList<EntityInfo>();
 		int arg = 1;
-		while (args.length >= arg + 2) {
+		while (args.length >= arg + 2)
+		{
 			EntityInfo eInfo = new EntityInfo(sentence,
 								   Integer.parseInt(args[arg]),
 								   Integer.parseInt(args[arg + 1]));
