@@ -1,4 +1,3 @@
-package relex.parser;
 /*
  * Copyright 2008 Novamente LLC
  *
@@ -15,6 +14,8 @@ package relex.parser;
  * limitations under the License.
  */
 
+package relex.parser;
+
 import org.linkgrammar.LinkGrammar;
 
 /**
@@ -27,7 +28,7 @@ import org.linkgrammar.LinkGrammar;
 
 public class LinkParserJNINewClient extends LinkParserClient
 {
-	private static final int verbosity = 1;
+	private static final int verbosity = 0;
 
 	private static LinkParserJNINewClient singletonInstance = null;
 
@@ -43,6 +44,13 @@ public class LinkParserJNINewClient extends LinkParserClient
 		return singletonInstance;
 	}
 
+	public String getVersion() {
+		// XXX hack alert -- this requires link-grammar version 4.3.6 or
+		// newer! So, for now, fake it.
+		// return LinkGrammar.getVersion();
+		return "link-grammar-4.3.5";
+	}
+
 	public void setMaxParseSeconds(int maxParseSeconds) {
 		super.setMaxParseSeconds(maxParseSeconds);
 		LinkGrammar.setMaxParseSeconds(maxParseSeconds);
@@ -71,15 +79,16 @@ public class LinkParserJNINewClient extends LinkParserClient
 
 	public void init(String linkGrammarDictionariesPath) {
 		super.init(linkGrammarDictionariesPath);
-		if (verbosity > 3) System.out.println("LinkParserJNINewClient: initializing.");
+		if (verbosity > 3) System.err.println("Info: LinkParserJNINewClient: initializing.");
 		LinkGrammar.init();
 	}
 
-	void execParse(String sentence) {
+	void execParse(String sentence)
+	{
 		Long starttime;
 		if (verbosity > 0)
 		{
-			if (verbosity > 3) System.out.println("parsing:" + sentence + "[end_sentence]");
+			if (verbosity > 3) System.err.println("Info: parsing:" + sentence + "[end_sentence]");
 			starttime = System.currentTimeMillis();
 		}
 		LinkGrammar.parse(sentence);
@@ -87,8 +96,8 @@ public class LinkParserJNINewClient extends LinkParserClient
 		{
 			Long now = System.currentTimeMillis();
 			Long elapsed = now - starttime;
-			System.out.println("Link parse time: " + elapsed + " milliseconds");
-			if (verbosity > 3) System.out.println("parsing LinkGrammar.completed.");
+			System.err.println("Info: Link parse time: " + elapsed + " milliseconds");
+			if (verbosity > 3) System.err.println("Info: Parsing LinkGrammar.completed.");
 		}
 	}
 
@@ -96,7 +105,8 @@ public class LinkParserJNINewClient extends LinkParserClient
 		return LinkGrammar.getNumLinkages();
 	}
 
-	void makeLinkage(int i) {
+	void makeLinkage(int i)
+	{
 		LinkGrammar.makeLinkage(i);
 	}
 
