@@ -405,10 +405,13 @@ public class PhraseTree
 	}
 
 	/**
-	 * Return the breadth of the tree
+	 * Return the breadth of the tree. This is equal to the number of
+	 * number of words in the tree.
 	 */
 	public static int getBreadth(FeatureNode word)
 	{
+		if (word == null) return 0;
+
 		int cnt = -1;  // don't count the phrase type as part of the breadth
 		FeatureNode fn = word.get("phr-head");
 		while (fn != null)
